/* 
 Copyright 2009 Foxmarks Inc.

 foxmarks-discover.js: Classes related to SERP and sim sites

 */

var Xmarks;
if(Xmarks === undefined){
    Xmarks = {};
}

(function() {
var xm = Xmarks;

xm.SimSites = {
    DRIFTSITE: "/site/?cid=xmfx&url=",
    SIMSITE: "/internal/visitsite?cid=xmfx&id=",
    TOPICSITE: "/topic/",
    WRITEREVIEW: "/review/edit/?cid=xmfx&url=",
    _iconLoaded: false,
    observe: function(subject, topic, data) {
        if(topic == "foxmarks-settingschange"){
            if(this.settings.simsiteEnabled && !this._iconLoaded){
                this.addIcon();
            } else if (!this.settings.simsiteEnabled &&
                this._iconLoaded){
                var image = this.doc.getElementById("foxmarks-ss-button");
                image.parentNode.removeChild(image);
                this._iconLoaded = false;
            }
        }
    },
    addIcon: function(){
        var staricon = this.doc.getElementById("star-button");
        if(staricon && !this._iconLoaded){
            var img = this.doc.createElement('image');
            img.setAttribute('id', "foxmarks-ss-button");
            img.setAttribute('tooltiptext',
                    xm.Bundle().GetStringFromName("similarsite.getsiteinfo"));
            img.setAttribute('src', 
                    xm.gSettings.httpProtocol + xm.gSettings.staticHost +
                    xm.gSettings.bibBug);
            img.addEventListener(
                "mousedown",
                function(e){
                    xm.SimSites.showSimSites();
                },
                true
            );

            staricon.parentNode.appendChild(img);
            this._iconLoaded = true;
        }
    },
    openLink: function(link, addTab){
        var callback= {
            notify: function(){
                if(addTab){
                    gBrowser.selectedTab = gBrowser.addTab(link);
                } else {
                    gBrowser.selectedBrowser.contentDocument.location.href =
                        link;
                }
            }
        };
        var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
        timer.initWithCallback(callback, 100,
            Ci.nsITimer.TYPE_ONE_SHOT);
    },
    init: function(){
        this.settings = xm.gSettings;
        this.doc = document;
        if(xm.gSettings.simsiteEnabled){
            this.addIcon();
        }
        var os = Cc["@mozilla.org/observer-service;1"].
            getService(Ci.nsIObserverService);
        os.addObserver(this, "foxmarks-settingschange", false);
        /*
         * FYI this doesn't work so well in linux so using our own 
         * messaging system
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefBranch2);
        prefs.addObserver("extensions.xmarks.", this, false);
        */
        var iframe = document.getElementById("foxmarks-bib");
        var self = this;
        var unloadFunc = function(event){
            iframe.removeEventListener(
                "unload",
                unloadFunc,
                true
            );
            var doc = event.target;
            var biblink = doc.getElementById("biblink");
            var link = biblink ? biblink.getAttribute("value") : null;

            if(link){
                var panel = document.getElementById("foxmarks-similarsites");
                if(panel){
                    panel.hidePopup();
                }
                var bibkeys = doc.getElementById("bibkeys");
                var keys = bibkeys.getAttribute("value");
                if(keys.indexOf("M") != -1 || keys.indexOf("C") != -1){
                    self.openLink(link, true);
                } else {
                    self.openLink(link, false);
                }
            }
        };
        iframe.addEventListener(
            "DOMContentLoaded", 
            function(event){
                if(event.target.location.href != "about:blank"){
                    self.handleBibLoad(event.target);
                    iframe.addEventListener(
                        "unload",
                        unloadFunc,
                        true
                    );
                }

            }, 
            true
        );
    },
    handleBibLoad: function(doc){
       var iframe = this.doc.getElementById("foxmarks-bib");
       var win = doc.defaultView;
       var pageHeight = win.innerHeight + win.scrollMaxY
       iframe.setAttribute("height",pageHeight);
       iframe.style.overflow = "hidden";
       this.doc.getElementById("foxmarks-ss-body").
            setAttribute("height", pageHeight);
       //iframe.style.height = "500px;";

    },
    clearPanel: function(){
        this.doc.getElementById("foxmarks-ss-progtitle").
            setAttribute("hidden", "true");
        this.doc.getElementById("foxmarks-ss-progress").
            setAttribute("hidden", "false");
        this.doc.getElementById("foxmarks-ss-body").
            setAttribute("hidden", "true");
       var iframe = this.doc.getElementById("foxmarks-bib");
        iframe.setAttribute("src", "about:blank");
       iframe.setAttribute("height","32");
       iframe.style.overflow = "hidden";
       this.doc.getElementById("foxmarks-ss-body").
            setAttribute("height", "32");
    },
    onPopupHiding: function(){
        if(this._inflight){
            this._inflight = false;
            Xmarks.SimilarSitesCancel();
        }
        xm.SimSites.detachKeyListener();
        return true;
    },
    _handleKeyListener: function(e){
        // static member; this is undefined
        if(e.metaKey){
           return;
        }
        var panel = this.doc.getElementById("foxmarks-similarsites");
        if(panel){
            panel.hidePopup();
        }
    },
    detachKeyListener: function(){
        var urlbar = this.doc.getElementById("urlbar");
        if(!urlbar){
            return;
        }
        urlbar.removeEventListener("keydown", this._handleKeyListener, true);
    },
    attachKeyListener: function(){
        var urlbar = this.doc.getElementById("urlbar");
        if(!urlbar){
            return;
        }
        urlbar.addEventListener("keydown", this._handleKeyListener, true);
    },
    showSimSites: function(){
        var panel = this.doc.getElementById("foxmarks-similarsites");
        if(panel){

            //BUGFIX for Windows  where popup would close randomly
            var content = this.doc.getElementById("content");
            content.focus();

            this.attachKeyListener();
            panel.openPopup(
                this.doc.getElementById("urlbar"),
                "after_end",
                0,
                0,
                false,
                false
            );
            var url = gBrowser.selectedBrowser.contentDocument.location.href;
            this.clearPanel();
            this._inflight = true;

            var self = this;
            Xmarks.FetchSimilarSites(
                url, 
                this.doc.getElementById("foxmarks-ss-progtitle"), 
                this.doc.getElementById("foxmarks-ss-throbber"), 
                function(data){
                    self._inflight = false;
                    xm.SimSites.loadData(data);
                }
            );
        }
    },
    loadData: function(data){

        if(data && data.status == 0){
            var iframe = this.doc.getElementById("foxmarks-bib");
            iframe.setAttribute("src", "data:text/html," + encodeURI(data.txt));
            this.doc.getElementById("foxmarks-ss-progress").
                setAttribute("hidden", "true");
            this.doc.getElementById("foxmarks-ss-body").
                setAttribute("hidden", "false");
        } else {
            var errbox = this.doc.getElementById("foxmarks-ss-progtitle");

            errbox.setAttribute("value",
                xm.Bundle().GetStringFromName(
                    data.status == 0 || data.status == 400 ?
                    "similarsite.nositeinfo" :
                    "similarsite.busy"
                )
            );
 
            errbox.setAttribute("hidden", "false");
        }
    }

};
xm.SERP = {
    DEBUGSCRIPTURL: "/SERP/gink.js",
    SCRIPTURL: "/SERP/",
    init: function(){
        var domain = xm.Bundle().GetStringFromName("serp.googleintl");
        this.INTL_PATTERN =
            new RegExp("http:\/\/" + domain + "\/.*[?&]q=([^&]+)");
        var appcontent = document.getElementById("appcontent");
        if(appcontent){
            appcontent.addEventListener(
                "DOMContentLoaded", 
                function(e){ 
                    if(e.originalTarget.nodeName == "#document"){
                        xm.SERP.onPageLoad(e);
                    }
                },
                true
            );
        }
    },
    buildScriptURL: function(){
        var lang = xm.gSettings.lang;
        if(xm.gSettings.getDebugOption("serp")){
            return this.DEBUGSCRIPTURL;
        }

        if(!lang){
            return this.SCRIPTURL + xm.gSettings.serpPrefix + ".js";
        }
        var langshort = lang.split('-')[0];
        
        if(langshort == "en"){
            return this.SCRIPTURL + xm.gSettings.serpPrefix + ".js";
        }
        var langs = {
            bn: 1, cs: 1, da: 1, de: 1, el: 1, eu: 1, es: 1, fi: 1, fr: 1,
            fy: 1, hu: 1, bg: 1, it: 1, ja: 1, ko: 1, nb: 1, nl: 1, pl: 1,
            ro: 1, ru: 1, sk: 1, sv: 1, tr: 1, uk: 1, vi: 1
        };

        if(langshort in langs){
            return "/SERP/" + xm.gSettings.serpPrefix + "." + langshort + ".js";
        }

        switch(lang){
            case "pt-BR":
            case "pt-PT":
            case "zh-CN":
            case "zh-TW":
                return "/SERP/" + xm.gSettings.serpPrefix + "." + lang + ".js";
        }

        return this.SCRIPTURL + xm.gSettings.serpPrefix + ".js";
    },
    onPageLoad: function(e){
        var doc = e.originalTarget;
        this.doc = doc;
        var rx = new RegExp(xm.gSettings.serpRegex);
        if(xm.gSettings.serpEnabled && doc && 
           doc.location && 
           doc.location.href && 
           rx.test(doc.location.href)
        ){
            doc = doc.defaultView.top.document;
            this.doc = doc;
            var bodytags = doc.getElementsByTagName('body');
            if(bodytags && bodytags.length == 1){
                var field = doc.createElement('input');
                field.setAttribute('type', 'hidden');
                field.setAttribute('id', 'foxmarks.prefs');
                var prefs = [];
                prefs.push("MACHINEID=" + xm.gSettings.machineId);
                prefs.push("NUM_TO_SHOW=" + xm.gSettings.serpMaxItems);
                prefs.push("APIHOST=" + xm.gSettings.apiHost);
                prefs.push("DRIFTHOST=" + xm.gSettings.driftHost);
                prefs.push("STATICHOST=" + xm.gSettings.staticHost);
                if(xm.gSettings.getDebugOption("serp")){
                    prefs.push("DEBUG=" + xm.gSettings.getDebugOption("serp"));
                }
                field.setAttribute('value', prefs.join(";"));
                bodytags[0].appendChild(field);

            }
            var headtags = doc.getElementsByTagName('head');
            if(headtags && headtags.length == 1){
                var script = doc.createElement('script');
                script.setAttribute('src', "http://"+ xm.gSettings.staticHost + 
                    this.buildScriptURL()
                );
                headtags[0].appendChild(script);

                this.insertST(doc);

            }
            if(doc.body){
                var self = this;
                doc.body.addEventListener(
                    "click",
                    function(e){
                        self.handleOpenSettings(e,doc);
                    },
                    false
                );
            }
            document.body
        }
    },
    handleOpenSettings: function(e, doc){
       if(e.target && e.target.id == "foxmarks.opensettings"){
            var os = Cc["@mozilla.org/observer-service;1"].
                getService(Ci.nsIObserverService);
            os.notifyObservers(null, 
                "foxmarks-showsettingpane", 
                "foxmarks-discoverypane"
            );
            e.stopPropagation();
       }
    },
    insertST: function(doc){
        var a = doc.getElementsByTagName("a");
        var len = a.length;
        var i;
        var self = this;

        this._stData = [];
        if(len == 0){
            return ;
        }
        var found = 10;
        for(var i =0; i < len; i++){
            var item = a[i];
            if(item.className == "l" &&
                item.parentNode && item.parentNode.className == "r" &&
                item.parentNode.nextSibling && 
                (item.parentNode.nextSibling.className== "s" ||
                item.parentNode.nextSibling.tagName == 'SPAN')){
                if(found > 0){
                    item.addEventListener("click", 
                        function(e){
                            self.handleClickST(e,doc);
                        },
                        true
                    );
                        
                    this._stData.push(item);
                    found--;
                } else {
                    break;
                }
            }
        }
        doc.defaultView.addEventListener("unload",
            function(e){
                self.handleUnloadST(e, doc);
            },
            true
        );

        doc.addEventListener("DOMNodeInserted",
            function(e){
                if(e.target && e.target.id == "foxmarks.arrow"){
                    self.handleHoverST(e, doc);

                }
            },
            false
        );
    },
    _isDec: function(obj){
        var gp = obj.parentNode.parentNode;
        var a = gp.getElementsByTagName('a');
        var len = a.length;
        var token = "foxmarks";

        for(var x = 0;  x < len ; x++){
            if(a[x].id && a[x].id.substr(0, token.length) == token){
                return true;
            }
        }

        return false;
    },
    _hasDec: function(){
        var a = this._stData;
        var len = a.length;
        var x;
        for(x = 0; x < len; x++){
           if(this._isDec(a[x])){
               return true;
           }
        }

        return false;
    },
    handleClickST: function(e, doc){
        var a = this._stData;
        var target = e.target;
        var len = a.length;
        var x;
        for(x = 0; x < len; x++){
           if(a[x] === target){
               break;
           }
        }
        if(x < len){
            xm.gSettings.incrST(x, this._isDec(target) ? "dc" : "uc",
                    this._hasDec());
        }

        
    },
    handleHoverST: function(e, doc){
        var target = e.target.parentNode.parentNode;
        var a = target.getElementsByTagName('a');
        var len = a.length;
        var item = null;
        var x;
        for(x = 0; x < len; x++){
           if(a[x].className == 'l'){
               item = a[x];
           }
        }
        if(item){
            a = this._stData;
            len = a.length;
            for(x = 0; x < len; x++){
                if(a[x] === item){
                    xm.gSettings.incrST(x, "h", true);
                    return;
                }
            }
        }
    },
    handleUnloadST: function(e, doc){
        var a = this._stData;
        var len = a.length;
        var x;
        var hasDec = this._hasDec();
        for(x = 0; x < len; x++){
            xm.gSettings.incrST(x, this._isDec(a[x]) ? "d" : "u", hasDec);
        }
        for(; x < 10; x++){
            xm.gSettings.incrST(x, "na", hasDec);
        }
        
        if(this.doc){
            var i = this.doc.getElementById("foxmarks.prefsout");
            if(i && i.value){
                xm.gSettings.trSERP = xm.gSettings.trSERP + i.value;
            }
        }
    }
};
})();

